==> nui_core-0.0.127.tar.gz: lib/busevent/nui_busevent.dart <==
import 'package:flutter/cupertino.dart';
import 'package:nui_core/nui_core.dart';

class NUIBusEventData<T> {
  final T data;
  final String code;
  final String sender;
  NUIBusEventData({this.data, this.sender, this.code});
}

abstract class NUIBusEventListener<T>{
  void onBusEventReceived(NUIBusEventData<T> payload);
}

typedef NUIBusEventListenerFunction<T> = void Function(NUIBusEventData<T> payload);

class NUIBusEvent {
  static const String _GLOBAL_CHANNEL = "nui_global_channel";
  static const String _DEFAULT_MODULE = "nui_default_module";
  static const String _MODULE = "NUIBusEvent";

  static Map<String, NUIBusEvent> _instance;

  Map<String, List<NUIBusEventListener>> _observers;
  Map<String, Map<String, NUIBusEventListenerFunction>> _observersInvoke;
  String _module;

  static NUIBusEvent get({String module}){
    final targetModule = module ?? _DEFAULT_MODULE;
    final instance = _instance[targetModule];
    if(instance == null){
      throwNUI(_MODULE, "Instance for module $targetModule have not been built with NUIBusEventBuilder");
    }
    return instance;
  }

  NUIBusEvent.builder(NUIBusEventBuilder builder){
    _module = builder._module;
    _observers = Map();
    _observersInvoke = Map();
    _instance = _instance ?? Map();
    _instance[_module ?? _DEFAULT_MODULE] = this;
  }

  void listen<T>(NUIBusEventListener<T> listener, {String channel}) {
    if(listener == null) return;

    final targetChannel = channel ?? _GLOBAL_CHANNEL;
    final observers = _observers[targetChannel] ?? List();
    observers.add(listener);
    _observers[targetChannel] = observers;
  }

  void listenInvoke<T>(String identity, {@required NUIBusEventListenerFunction<T> listener, String channel}) {
    if(listener == null) return;

    final targetChannel = channel ?? _GLOBAL_CHANNEL;
    final observers = _observersInvoke[targetChannel] ?? Map();
    observers[identity] = listener;
    _observersInvoke[targetChannel] = observers;
  }

  void send<T>(NUIBusEventData<T> payload, {String channel}) {
    final targetChannel = channel ?? _GLOBAL_CHANNEL;
    final observers = _observers[targetChannel] ?? List();
    final observersInvoke = _observersInvoke[targetChannel] ?? Map();
    observers.forEach((NUIBusEventListener obj){
      try {
        if(obj is NUIBusEventListener<T>) {
          obj.onBusEventReceived(payload);
        }
      }catch(e){
        logNUI(_MODULE, "Exception encountered on sending bus event with error : $e");
      }
    });

    for(var func in observersInvoke.values){
      try {
        if(func is NUIBusEventListenerFunction<T>) {
          func(payload);
        }
      }catch(e){
        logNUI(_MODULE, "Exception encountered on sending bus event with error : $e");
      }
    }
  }

  void clear({String channel}){
    final targetChannel = channel ?? _GLOBAL_CHANNEL;
    final observers = _observers[targetChannel] ?? List();
    final observersInvoke = _observersInvoke[targetChannel] ?? Map();
    observers.clear();
    observersInvoke.clear();
    _observers[targetChannel] = observers;
    _observersInvoke[targetChannel] = observersInvoke;
  }

  void dispose<T>(NUIBusEventListener<T> observer, {String channel}) {
    final targetChannel = channel ?? _GLOBAL_CHANNEL;
    final observers = _observers[targetChannel] ?? List();
    observers.remove(observer);
    _observers[targetChannel] = observers;
  }

  void disposeInvoke(String identity, {String channel}) {
    final targetChannel = channel ?? _GLOBAL_CHANNEL;
    final observersInvoke = _observersInvoke[targetChannel] ?? Map();
    _observersInvoke.remove(identity);
    _observersInvoke[targetChannel] = observersInvoke;
  }
}

class NUIBusEventBuilder{
  String _module;

  NUIBusEventBuilder module(String module){
    this._module = module;
    return this;
  }

  NUIBusEvent build(){
    return NUIBusEvent.builder(this);
  }
}

//abstract class NUIBusEvent {
//  /// Send [data] as broadcasts to listeners registered with [action].
//  void notify<T>(String action, T data);
//
//  /// Register a new callback on [action].
//  Widget register<T>(String action, Widget Function(NUIBusEventData<T>) callback);
//
//  /// Dispose any resources that are open.
//  void dispose();
//
//  static NUIBusEvent of(BuildContext context) {
//    return NUIBusEventProvider.of(context);
//  }
//}

///// Implementation of [NUIBusEvent].
/////
///// Uses [Stream] as primary way to communicate broadcasts.
///// Callbacks are wrapped inside a [StreamBuilder] which is returned from [build]
///// method of a [StatefulWidget]. Whenever [notify] is called, the stream
///// is notified for changes, so all the [StreamBuilder]'s connected to that stream
///// are rebuilt thus fetching new data.
/////
///// Every operation is added to a queue of work using [Lock] from
///// 'synchronized' package.
//class NUIBusEventImpl implements NUIBusEvent {
//  // Main [Lock] used for every operation
//  final _lock = Lock();
//
//  /// Mapping of current number of callbacks registered with a certain action.
//  Map<String, int> _registerCount = {};
//
//  /// Mapping of current data for a certain action;
//  Map<String, dynamic> _actionDataMappings = {};
//
//  /// Mapping of action and the stream opened for the same.
//  Map<String, StreamController<dynamic>> _actionStreamMappings = {};
//
//  static NUIBusEvent of(BuildContext context) {
//    return NUIBusEventProvider.of(context);
//  }
//
//  @override
//  void notify<T>(String action, T data) {
//    _lock.synchronized(() {
//      // Update the action-data mappings.
//      _actionDataMappings[action] = NUIBusEventData(hasData: data != null, data: data);
//
//      // Notify the stream that data has been changed.
//      _actionStreamMappings[action]?.add(null);
//    });
//  }
//
//  @override
//  Widget register<T>(String action, Widget Function(NUIBusEventData<T>) callback) {
//    _checkAndCreateStream(action);
//
//    return _BusEventWidget(
//      action: action,
//      callback: callback,
//      notifier: this,
//    );
//  }
//
//  @override
//  void dispose() {
//    _actionStreamMappings.values.forEach((stream) => stream.close());
//  }
//
//  /// Checks if there a stream for a particular [action], if not then a new
//  /// [Stream] is created for that action and default data is added to it;
//  ///
//  /// This functions helps to prevent adding [Stream] for actions that don't
//  /// have any callbacks.
//  void _checkAndCreateStream(String action) {
//    if (!_actionStreamMappings.containsKey(action)) {
//      _actionStreamMappings[action] = StreamController.broadcast();
//      _actionDataMappings[action] = NUIBusEventData(hasData: false, data: null);
//    }
//  }
//
//  /// Increment the current count of callbacks registered for the stream
//  /// with [action].
//  void _incrementRegisterCount(String action) {
//    _lock.synchronized(() {
//      _registerCount[action] = (_registerCount[action] ?? 0) + 1;
//    });
//  }
//
//  /// Decrement the current count of callbacks registered for the stream
//  /// with [action].
//  ///
//  /// Check if it was the last callback. If so then close and remove the stream
//  /// from mappings.
//  void _decrementRegisterCount(String action) {
//    _lock.synchronized(() {
//      _registerCount[action] = (_registerCount[action] ?? 0) - 1;
//
//      if (_registerCount[action] <= 0) {
//        _actionStreamMappings[action]?.close();
//
//        _registerCount.remove(action);
//        _actionStreamMappings.remove(action);
//        _actionDataMappings.remove(action);
//      }
//    });
//  }
//}
//
///// Widget that wraps the callback into a [StreamBuilder] for a particular
///// action.
//class _BusEventWidget extends StatefulWidget {
//  final String action;
//  final dynamic callback;
//  final NUIBusEventImpl notifier;
//
//  const _BusEventWidget({
//    Key key,
//    @required this.action,
//    @required this.callback,
//    @required this.notifier,
//  }) : super(key: key);
//
//  @override
//  _BusEventWidgetState createState() => _BusEventWidgetState();
//}
//
//class _BusEventWidgetState extends State<_BusEventWidget> {
//  @override
//  void initState() {
//    widget.notifier._incrementRegisterCount(widget.action);
//    super.initState();
//  }
//
//  @override
//  void dispose() {
//    widget.notifier._decrementRegisterCount(widget.action);
//    super.dispose();
//  }
//
//  @override
//  Widget build(BuildContext context) {
//    return StreamBuilder(
//      stream: widget.notifier._actionStreamMappings[widget.action].stream
//          .asBroadcastStream(),
//      builder: (context, _) {
//        final data = widget.notifier._actionDataMappings[widget.action];
//        return widget.callback(data);
//      },
//    );
//  }
//}
//
//class NUIBusEventProvider extends InheritedWidget {
//  final NUIBusEvent _notifier = NUIBusEventImpl();
//
//  NUIBusEventProvider({Widget child}) : super(child: child);
//
//  @override
//  bool updateShouldNotify(InheritedWidget oldWidget) {
//    return true;
//  }
//
//  static NUIBusEvent of(BuildContext context) {
//    return (context.inheritFromWidgetOfExactType(NUIBusEventProvider)
//    as NUIBusEventProvider)
//        ._notifier;
//  }
//}

==> nui_core-0.0.250.tar.gz: lib/busevent/nui_busevent.dart <==
import 'package:flutter/cupertino.dart';
import 'package:nui_core/nui_core.dart';

class NUIBusEventData<T> {
  final T data;
  final String code;
  final String sender;
  NUIBusEventData({this.data, this.sender, this.code});
}

abstract class NUIBusEventListener<T>{
  void onBusEventReceived(NUIBusEventData<T> payload);
}

typedef NUIBusEventListenerFunction<T> = void Function(NUIBusEventData<T> payload);

class NUIBusEvent {
  static const String _GLOBAL_CHANNEL = "nui_global_channel";
  static const String _DEFAULT_MODULE = "nui_default_module";
  static const String _MODULE = "NUIBusEvent";

  static Map<String, NUIBusEvent> _instance;

  Map<String, List<NUIBusEventListener>> _observers;
  Map<String, Map<String, NUIBusEventListenerFunction>> _observersInvoke;
  String _module;

  static NUIBusEvent get({String module}){
    final targetModule = module ?? _DEFAULT_MODULE;
    final instance = _instance[targetModule];
    if(instance == null){
      throwNUI(_MODULE, "Instance for module $targetModule have not been built with NUIBusEventBuilder");
    }
    return instance;
  }

  NUIBusEvent.builder(NUIBusEventBuilder builder){
    _module = builder._module;
    _observers = Map();
    _observersInvoke = Map();
    _instance = _instance ?? Map();
    _instance[_module ?? _DEFAULT_MODULE] = this;
  }

  void listen<T>(NUIBusEventListener<T> listener, {String channel}) {
    if(listener == null) return;

    final targetChannel = channel ?? _GLOBAL_CHANNEL;
    final observers = _observers[targetChannel] ?? List();
    observers.add(listener);
    _observers[targetChannel] = observers;
  }

  void listenInvoke<T>(String identity, {@required NUIBusEventListenerFunction<T> listener, String channel}) {
    if(listener == null) return;

    final targetChannel = channel ?? _GLOBAL_CHANNEL;
    final observers = _observersInvoke[targetChannel] ?? Map();
    observers[identity] = listener;
    _observersInvoke[targetChannel] = observers;
  }

  void send<T>(NUIBusEventData<T> payload, {String channel}) {
    final targetChannel = channel ?? _GLOBAL_CHANNEL;
    final observers = _observers[targetChannel] ?? [];
    final observersInvoke = _observersInvoke[targetChannel] ?? Map();
    observers.forEach((NUIBusEventListener obj){
      // if(obj is NUIBusEventListener<T>) {
      try {
        obj.onBusEventReceived(payload);
      } catch(e){
        logNUI(_MODULE, "Exception encountered on sending bus event with error : $e");
      }
      // }
    });

    for(var func in observersInvoke.values){
      try {
        if(func is NUIBusEventListenerFunction<T>) {
          func(payload);
        }
      }catch(e){
        logNUI(_MODULE, "Exception encountered on sending bus event with error : $e");
      }
    }
  }

  void clear({String channel}){
    final targetChannel = channel ?? _GLOBAL_CHANNEL;
    final observers = _observers[targetChannel] ?? List();
    final observersInvoke = _observersInvoke[targetChannel] ?? Map();
    observers.clear();
    observersInvoke.clear();
    _observers[targetChannel] = observers;
    _observersInvoke[targetChannel] = observersInvoke;
  }

  void dispose<T>(NUIBusEventListener<T> observer, {String channel}) {
    final targetChannel = channel ?? _GLOBAL_CHANNEL;
    final observers = _observers[targetChannel] ?? List();
    observers.remove(observer);
    _observers[targetChannel] = observers;
  }

  void disposeInvoke(String identity, {String channel}) {
    final targetChannel = channel ?? _GLOBAL_CHANNEL;
    final observersInvoke = _observersInvoke[targetChannel] ?? Map();
    _observersInvoke.remove(identity);
    _observersInvoke[targetChannel] = observersInvoke;
  }
}

class NUIBusEventBuilder{
  String _module;

  NUIBusEventBuilder module(String module){
    this._module = module;
    return this;
  }

  NUIBusEvent build(){
    return NUIBusEvent.builder(this);
  }
}

//abstract class NUIBusEvent {
//  /// Send [data] as broadcasts to listeners registered with [action].
//  void notify<T>(String action, T data);
//
//  /// Register a new callback on [action].
//  Widget register<T>(String action, Widget Function(NUIBusEventData<T>) callback);
//
//  /// Dispose any resources that are open.
//  void dispose();
//
//  static NUIBusEvent of(BuildContext context) {
//    return NUIBusEventProvider.of(context);
//  }
//}

///// Implementation of [NUIBusEvent].
/////
///// Uses [Stream] as primary way to communicate broadcasts.
///// Callbacks are wrapped inside a [StreamBuilder] which is returned from [build]
///// method of a [StatefulWidget]. Whenever [notify] is called, the stream
///// is notified for changes, so all the [StreamBuilder]'s connected to that stream
///// are rebuilt thus fetching new data.
/////
///// Every operation is added to a queue of work using [Lock] from
///// 'synchronized' package.
//class NUIBusEventImpl implements NUIBusEvent {
//  // Main [Lock] used for every operation
//  final _lock = Lock();
//
//  /// Mapping of current number of callbacks registered with a certain action.
//  Map<String, int> _registerCount = {};
//
//  /// Mapping of current data for a certain action;
//  Map<String, dynamic> _actionDataMappings = {};
//
//  /// Mapping of action and the stream opened for the same.
//  Map<String, StreamController<dynamic>> _actionStreamMappings = {};
//
//  static NUIBusEvent of(BuildContext context) {
//    return NUIBusEventProvider.of(context);
//  }
//
//  @override
//  void notify<T>(String action, T data) {
//    _lock.synchronized(() {
//      // Update the action-data mappings.
//      _actionDataMappings[action] = NUIBusEventData(hasData: data != null, data: data);
//
//      // Notify the stream that data has been changed.
//      _actionStreamMappings[action]?.add(null);
//    });
//  }
//
//  @override
//  Widget register<T>(String action, Widget Function(NUIBusEventData<T>) callback) {
//    _checkAndCreateStream(action);
//
//    return _BusEventWidget(
//      action: action,
//      callback: callback,
//      notifier: this,
//    );
//  }
//
//  @override
//  void dispose() {
//    _actionStreamMappings.values.forEach((stream) => stream.close());
//  }
//
//  /// Checks if there a stream for a particular [action], if not then a new
//  /// [Stream] is created for that action and default data is added to it;
//  ///
//  /// This functions helps to prevent adding [Stream] for actions that don't
//  /// have any callbacks.
//  void _checkAndCreateStream(String action) {
//    if (!_actionStreamMappings.containsKey(action)) {
//      _actionStreamMappings[action] = StreamController.broadcast();
//      _actionDataMappings[action] = NUIBusEventData(hasData: false, data: null);
//    }
//  }
//
//  /// Increment the current count of callbacks registered for the stream
//  /// with [action].
//  void _incrementRegisterCount(String action) {
//    _lock.synchronized(() {
//      _registerCount[action] = (_registerCount[action] ?? 0) + 1;
//    });
//  }
//
//  /// Decrement the current count of callbacks registered for the stream
//  /// with [action].
//  ///
//  /// Check if it was the last callback. If so then close and remove the stream
//  /// from mappings.
//  void _decrementRegisterCount(String action) {
//    _lock.synchronized(() {
//      _registerCount[action] = (_registerCount[action] ?? 0) - 1;
//
//      if (_registerCount[action] <= 0) {
//        _actionStreamMappings[action]?.close();
//
//        _registerCount.remove(action);
//        _actionStreamMappings.remove(action);
//        _actionDataMappings.remove(action);
//      }
//    });
//  }
//}
//
///// Widget that wraps the callback into a [StreamBuilder] for a particular
///// action.
//class _BusEventWidget extends StatefulWidget {
//  final String action;
//  final dynamic callback;
//  final NUIBusEventImpl notifier;
//
//  const _BusEventWidget({
//    Key key,
//    @required this.action,
//    @required this.callback,
//    @required this.notifier,
//  }) : super(key: key);
//
//  @override
//  _BusEventWidgetState createState() => _BusEventWidgetState();
//}
//
//class _BusEventWidgetState extends State<_BusEventWidget> {
//  @override
//  void initState() {
//    widget.notifier._incrementRegisterCount(widget.action);
//    super.initState();
//  }
//
//  @override
//  void dispose() {
//    widget.notifier._decrementRegisterCount(widget.action);
//    super.dispose();
//  }
//
//  @override
//  Widget build(BuildContext context) {
//    return StreamBuilder(
//      stream: widget.notifier._actionStreamMappings[widget.action].stream
//          .asBroadcastStream(),
//      builder: (context, _) {
//        final data = widget.notifier._actionDataMappings[widget.action];
//        return widget.callback(data);
//      },
//    );
//  }
//}
//
//class NUIBusEventProvider extends InheritedWidget {
//  final NUIBusEvent _notifier = NUIBusEventImpl();
//
//  NUIBusEventProvider({Widget child}) : super(child: child);
//
//  @override
//  bool updateShouldNotify(InheritedWidget oldWidget) {
//    return true;
//  }
//
//  static NUIBusEvent of(BuildContext context) {
//    return (context.inheritFromWidgetOfExactType(NUIBusEventProvider)
//    as NUIBusEventProvider)
//        ._notifier;
//  }
//}
